# Generated by roxygen2: do not edit by hand

export(bad_arg)
export(convert_MM_vector_to_locations)
export(convert_base_to_number)
export(convert_locations_to_MM_vector)
export(convert_modification_to_number_vector)
export(convert_sequence_to_numbers)
export(convert_sequences_to_matrix)
export(create_image_data)
export(debug_join_vector_num)
export(debug_join_vector_str)
export(extract_and_sort_methylation)
export(extract_and_sort_sequences)
export(extract_methylation_from_dataframe)
export(extract_sequences_from_dataframe)
export(format_time_diff)
export(ggDNAvis_shiny)
export(ggDNAvis_shinyapp)
export(insert_at_indices)
export(merge_fastq_with_metadata)
export(merge_methylation_with_metadata)
export(monitor)
export(monitor_start)
export(rasterise_index_annotations)
export(rasterise_matrix)
export(rasterise_probabilities)
export(rasterize_index_annotations)
export(rasterize_matrix)
export(rasterize_probabilities)
export(read_fastq)
export(read_modified_fastq)
export(resolve_alias)
export(resolve_alias_map)
export(reverse_complement)
export(reverse_locations_if_needed)
export(reverse_probabilities_if_needed)
export(reverse_quality_if_needed)
export(reverse_sequence_if_needed)
export(string_to_vector)
export(strip_leading_at)
export(vector_to_string)
export(visualise_many_sequences)
export(visualise_methylation)
export(visualise_methylation_colour_scale)
export(visualise_single_sequence)
export(visualize_many_sequences)
export(visualize_methylation)
export(visualize_methylation_color_scale)
export(visualize_single_sequence)
export(write_fastq)
export(write_modified_fastq)
import(dplyr)
import(ggplot2)
importFrom(cli,cli_alert_info)
importFrom(ggnewscale,new_scale_fill)
importFrom(grid,grid.newpage)
importFrom(grid,grid.raster)
importFrom(magick,image_compare)
importFrom(magick,image_read)
importFrom(png,readPNG)
importFrom(rlang,abort)
importFrom(rlang,warn)
importFrom(stats,setNames)
importFrom(stringr,str_locate_all)
importFrom(tidyr,drop_na)
importFrom(tidyr,replace_na)
importFrom(utils,head)
importFrom(utils,packageDescription)
importFrom(utils,tail)
