This software is freely available under the open MIT licence, but please make sure
to cite the [ggDNAvis paper][link_citation] when using in publications or presentations.

Please report any bugs, difficulties, or feature requests to the [issues page][link_bugs].

These interactive tools provide most of the functionality of the [full R package][link_CRAN],
and enable a level of reproducibility via the settings import/export options.
However, using the full R package via re-runable scripts improves reproducibility,
as well as enabling advanced usage such as annotating returned ggplot2 objects
(see [example of adding ggplot2 elements to visualisation][link_methylation_offset]).
To get started with the code-based R package, read the [documentation][link_documentation].

From an R session with ggDNAvis installed, a local version of this Shiny application can
be launched via `ggDNAvis_shinyapp()`.
