---
title: "Interactive App"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Interactive App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

<style>
    html, body {
        overflow: hidden !important;
        height: 100%;
        width: 100%;
        padding-bottom: 0 !important;
    }
    
    body > .container-template-article, 
    body > .container-fluid, 
    body > .container {
        max-width: 100% !important;
        width: 100% !important;
        padding: 0 !important;
        margin: 0 !important;
        height: 100%;
        display: flex;
        flex-direction: column;
    }
    .navbar {
        position: relative !important;
        top: auto !important;
        margin-bottom: 0 !important;
        width: 100% !important;
        z-index: 1000;
    }
    .row {
        margin-left: 0 !important;
        margin-right: 0 !important;
    }
    .row > main, .col-md-9 {
        padding: 0 !important;
        margin: 0 !important;
        width: 100% !important;
        max-width: 100% !important;
        flex: 1 0 auto;
        
    }
    .page-header, .pkgdown-toc, .col-md-3, footer {
        display: none;
    }
</style>

<iframe 
    src="https://ejade42-ggdnavis.share.connect.posit.cloud/" 
    width="100%" 
    data-external="1" 
    style="height: calc(100vh - 50px); border: none; display: block;">
</iframe>
