**Supported formats:**

 * **FASTA:** Standard format starting with a `'>'` header line, 
followed by the sequence. The sequence is allowed to be on one 
or multiple lines after the header line. 
If multiple sequences are present they will be concatenated.

 * **Plain text:** A simple text file containing only the DNA 
 sequence on one or multiple lines. Lines will be concatenated.

**Allowed characters:**

 * A, C, G, T, and U (uppercase or lowercase)
 
 * Whitespace will be removed (spaces, tabs, and linebreaks)
